#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <glib.h>
#ifdef HAVE_GTK20
#	include <gdk-pixbuf/gdk-pixbuf.h>
#endif

#include <g3dviewer_plugin.h>

int plugin_load(const char *filename, G3DImage *image)
{
#ifdef HAVE_GTK20
	GdkPixbuf *pixbuf;
	guint32 x, y;
	guchar *p;

	pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
	if(pixbuf == NULL)
	{
		g_warning("failed to load '%s'", filename);
		return EXIT_FAILURE;
	}

	if(gdk_pixbuf_get_colorspace(pixbuf) != GDK_COLORSPACE_RGB)
	{
		g_warning("GdkPixbuf: %s: colorspace is not RGB", filename);
		gdk_pixbuf_unref(pixbuf);
		return EXIT_FAILURE;
	}

	if(gdk_pixbuf_get_n_channels(pixbuf) < 3)
	{
		g_warning("GdkPixbuf: %s: has only %d channels", filename,
			gdk_pixbuf_get_n_channels(pixbuf));
		gdk_pixbuf_unref(pixbuf);
		return EXIT_FAILURE;
	}

	image->width = gdk_pixbuf_get_width(pixbuf);
	image->height = gdk_pixbuf_get_height(pixbuf);
	image->depth = 32;
	image->name = g_path_get_basename(filename);
	image->pixeldata = g_new0(guint8, image->width * image->height * 4);

	for(y = 0; y < image->height; y ++)
		for(x = 0; x < image->width; x ++)
		{
			p = gdk_pixbuf_get_pixels(pixbuf) + y *
				gdk_pixbuf_get_rowstride(pixbuf) + x * 4;

			image->pixeldata[(y * image->width + x) * 4 + 0] = p[0];
			image->pixeldata[(y * image->width + x) * 4 + 1] = p[1];
			image->pixeldata[(y * image->width + x) * 4 + 2] = p[2];
			image->pixeldata[(y * image->width + x) * 4 + 3] = p[3];
		}

	/* set alpha to 1.0 */
	if(gdk_pixbuf_get_n_channels(pixbuf) < 4)
		for(y = 0; y < image->height; y ++)
			for(x = 0; x < image->width; x ++)
				image->pixeldata[(y * image->width + x) * 4 + 3] = 0xFF;

	gdk_pixbuf_unref(pixbuf);

#if DEBUG > 0
	g_print("GdkPixbuf: image '%s' loaded (%dx%d)\n",
		image->name, image->width, image->height);
#endif

	return EXIT_SUCCESS;
#else
	return EXIT_FAILURE;
#endif
}

char *plugin_description(void)
{
	return g_strdup("plugin to read images using the GdkPixbuf library");
}

char **plugin_extensions(void)
{
	gchar *extensions = g_strdup("");
#ifdef HAVE_GTK20
	gchar *tmp;
	GSList *fitem;
	GdkPixbufFormat *format;

	fitem = gdk_pixbuf_get_formats();
	while(fitem)
	{
		format = (GdkPixbufFormat *)fitem->data;
		tmp = g_strdup_printf("%s%s%s", extensions,
			strlen(extensions) ? ":" : "",
			g_strjoinv(":", gdk_pixbuf_format_get_extensions(format)));
		g_free(extensions);
		extensions = tmp;
		fitem = fitem->next;
	}
#endif
	return g_strsplit(extensions, ":", 0);
}
