#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <g3dviewer_plugin.h>

int image_expand(G3DImage *image, guint newwidth, guint newheight)
{
	guint32 y;
	guint8 *newptr;

	g_return_val_if_fail(image != NULL, EXIT_FAILURE);
	g_return_val_if_fail(image->width <= newwidth, EXIT_FAILURE);
	g_return_val_if_fail(image->height <= newheight, EXIT_FAILURE);

	newptr = g_new0(guint8, newwidth * newheight * 4);

	if(newptr == NULL)
	{
		g_printerr("ERROR: image_expand: resizing failed (to %ux%ux%ubit)\n",
			newwidth, newheight, image->depth);
		return EXIT_FAILURE;
	}

	for(y = 0; y < image->height; y ++)
	{
		memcpy(
			&(newptr[y * newwidth * 4]),
			&(image->pixeldata[y * image->width * 4]),
			image->width * 4);
	}

#if 0
	g_free(image->pixeldata);
#endif

	image->pixeldata = newptr;
	image->width = newwidth;
	image->height = newheight;

	return EXIT_SUCCESS;
}

int g3d_image_dump_ppm(G3DImage *image, const gchar *filename)
{
	FILE *f;
	guint32 x, y;

	f = fopen(filename, "w");
	if(f == NULL)
	{
		g_warning("image: failed to write to '%s'", filename);
		return EXIT_FAILURE;
	}

	fprintf(f, "P3\n# CREATOR: g3dviewer\n%d %d\n%d\n",
		image->width, image->height, 255);

	for(y = 0; y < image->height; y ++)
	{
		for(x = 0; x < image->width; x ++)
		{
			fprintf(f, "%d\n%d\n%d\n",
				image->pixeldata[(y * image->width + x) * 4 + 0],
				image->pixeldata[(y * image->width + x) * 4 + 1],
				image->pixeldata[(y * image->width + x) * 4 + 2]);
		}
	}

	fclose(f);

	return EXIT_SUCCESS;
}
