#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <stdlib.h>

#include <gtk/gtk.h>

#ifdef HAVE_GTKGLEXT
#	include <gtk/gtkgl.h>
#else
#	include <gtkgl/gtkglarea.h>
#endif

#include <g3dviewer_plugin.h>

#include "interface.h"
#include "infowin.h"
#include "object.h"
#include "plugins.h"
#include "glarea.h"
#include "gl.h"

GtkWidget *interface_create_glarea(G3DModel *model);
GtkWidget *interface_create_menu(G3DModel *model);

int interface_init(G3DModel *model)
{
	GtkWidget *vbox, *popupmenu;

	/* window */
	model->interface.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect_object(GTK_OBJECT(model->interface.window),
		"delete-event", gtk_main_quit, (gpointer)model->interface.window);
	gtk_window_set_default_size(GTK_WINDOW(model->interface.window), 400, 440);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(model->interface.window), vbox);

	/* glarea */
	model->interface.glarea = interface_create_glarea(model);
	gtk_box_pack_start(GTK_BOX(vbox), model->interface.glarea, TRUE, TRUE, 0);

	/* status bar */
	model->interface.statushbox = gtk_hbox_new(FALSE, 1);
	model->interface.progressbar = gtk_progress_bar_new();
	gtk_progress_bar_set_bar_style(
		GTK_PROGRESS_BAR(model->interface.progressbar),
		GTK_PROGRESS_LEFT_TO_RIGHT);
	gtk_box_pack_start(GTK_BOX(model->interface.statushbox),
		model->interface.progressbar, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), model->interface.statushbox,
		FALSE, TRUE, 0);

	gtk_widget_show_all(model->interface.window);

	model->interface.infowin = infowin_create(model);

	/* popup menu */
	popupmenu = interface_create_menu(model);
	gtk_object_set_data(GTK_OBJECT(model->interface.glarea), "menu",
		popupmenu);

	if(model->interface.show_statusbar == 0)
	{
		gtk_widget_hide(model->interface.statushbox);
	}

	return EXIT_SUCCESS;
}

GtkWidget *interface_create_glarea(G3DModel *model)
{
	GtkWidget *glarea;
#ifdef HAVE_GTKGLEXT
	GdkGLConfig *glconfig;
#endif

#ifdef HAVE_GTKGLEXT
	glconfig = gdk_gl_config_new_by_mode(
		GDK_GL_MODE_RGB | GDK_GL_MODE_DEPTH | GDK_GL_MODE_DOUBLE);

	if(glconfig == NULL)
	{
		g_printerr("can't get double-buffered visual\n");
		glconfig = gdk_gl_config_new_by_mode(
			GDK_GL_MODE_RGBA | GDK_GL_MODE_DEPTH |
			GDK_GL_MODE_ALPHA | GDK_GL_MODE_DOUBLE);
	}

	glarea = gtk_drawing_area_new();

	gtk_widget_set_gl_capability(glarea, glconfig, NULL, TRUE,
		GDK_GL_RGBA_TYPE);

#else
	glarea = gtk_gl_area_new_vargs(NULL,
		GDK_GL_RGBA,
		GDK_GL_RED_SIZE, 1,
		GDK_GL_GREEN_SIZE, 1,
		GDK_GL_BLUE_SIZE, 1,
		GDK_GL_DEPTH_SIZE, 1,
		GDK_GL_DOUBLEBUFFER,
		GDK_GL_NONE);
#endif

	if(glarea == NULL) return NULL;

	gtk_widget_set_events(glarea,
		GDK_EXPOSURE_MASK |
		GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
#ifdef HAVE_GTK20
		GDK_SCROLL_MASK |
#endif
		GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);

#ifdef HAVE_GTK20
	gtk_signal_connect(GTK_OBJECT(glarea), "scroll_event",
		GTK_SIGNAL_FUNC(glarea_scroll), NULL);
#endif

	gtk_signal_connect(GTK_OBJECT(glarea), "expose_event",
		GTK_SIGNAL_FUNC(glarea_expose), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "motion_notify_event",
		GTK_SIGNAL_FUNC(glarea_motion_notify), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "button_press_event",
		GTK_SIGNAL_FUNC(glarea_button_pressed), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "configure_event",
		GTK_SIGNAL_FUNC(glarea_configure), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "destroy",
		GTK_SIGNAL_FUNC(glarea_destroy), NULL);

	gtk_object_set_data(GTK_OBJECT(glarea), "model", model);

	return glarea;
}

void interface_open_cb(GtkWidget *widget, gpointer user_data)
{
	interface_showopendialog((G3DModel*)user_data);
}

void interface_selectcolor_cb(GtkWidget *widget, gpointer user_data)
{
	GtkColorSelectionDialog *dialog = (GtkColorSelectionDialog*)user_data;
	G3DModel *model = gtk_object_get_data(GTK_OBJECT(dialog), "model");
	gdouble color[4] = {0.9, 0.8, 0.6, 1.0};

	gtk_color_selection_get_color(
		GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel),
		color);

	model->bgcolor[0] = color[0];
	model->bgcolor[1] = color[1];
	model->bgcolor[2] = color[2];
	model->bgcolor[3] = color[3];
}

void interface_color_cb(GtkWidget *widget, gpointer user_data)
{
	G3DModel *model = (G3DModel*)user_data;
	GtkWidget *dialog;
	gdouble color[4];

	dialog = gtk_color_selection_dialog_new("Background color...");
	gtk_object_set_data(GTK_OBJECT(dialog), "model", model);

	/* GLfloat[] => gdouble[] */
	color[0] = model->bgcolor[0];
	color[1] = model->bgcolor[1];
	color[2] = model->bgcolor[2];
	color[3] = model->bgcolor[3];

	gtk_color_selection_set_color(
		GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel),
		color);
	gtk_signal_connect(
		GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(dialog)->ok_button), "clicked",
		GTK_SIGNAL_FUNC(interface_selectcolor_cb), (gpointer)dialog);
	gtk_signal_connect_object_after(
		GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(dialog)->ok_button), "clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);
	gtk_signal_connect_object_after(
		GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(dialog)->cancel_button),
		"clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);

	gtk_widget_show(dialog);
}

void interface_togglemode_cb(GtkWidget *widget, gpointer user_data)
{
	int wf = GTK_CHECK_MENU_ITEM(widget)->active;
	if(wf == 1)
	{
		glPolygonMode(GL_FRONT, GL_LINE);
		glPolygonMode(GL_BACK, GL_LINE);
	}
	else
	{
		glPolygonMode(GL_FRONT, GL_FILL);
		glPolygonMode(GL_BACK, GL_FILL);
	}
#if DEBUG > 3
	g_printerr("interface_togglemode_cb: Wireframe: %d\n", wf);
#endif
}

void interface_togglespecular_cb(GtkWidget *widget, gpointer user_data)
{
	G3DModel *model = (G3DModel*)user_data;
	if(GTK_CHECK_MENU_ITEM(widget)->active)
		model->glflags |= G3D_FLAG_GL_SPECULAR;
	else
		model->glflags &= ~G3D_FLAG_GL_SPECULAR;
}

void interface_toggleshininess_cb(GtkWidget *widget, gpointer user_data)
{
	G3DModel *model = (G3DModel*)user_data;
	if(GTK_CHECK_MENU_ITEM(widget)->active)
		model->glflags |= G3D_FLAG_GL_SHININESS;
	else
		model->glflags &= ~G3D_FLAG_GL_SHININESS;
}

void interface_toggletwosided_cb(GtkWidget *widget, gpointer user_data)
{
	G3DModel *model = (G3DModel*)user_data;
	if(GTK_CHECK_MENU_ITEM(widget)->active)
		model->glflags |= G3D_FLAG_GL_ALLTWOSIDE;
	else
		model->glflags &= ~G3D_FLAG_GL_ALLTWOSIDE;

	gl_set_twoside(model->glflags & G3D_FLAG_GL_ALLTWOSIDE);
}

void interface_toggletextures_cb(GtkWidget *widget, gpointer user_data)
{
	G3DModel *model = (G3DModel*)user_data;
	if(GTK_CHECK_MENU_ITEM(widget)->active)
		model->glflags |= G3D_FLAG_GL_TEXTURES;
	else
		model->glflags &= ~G3D_FLAG_GL_TEXTURES;

	gl_set_textures(GTK_CHECK_MENU_ITEM(widget)->active);
}

GtkWidget *interface_create_menu(G3DModel *model)
{
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *mitem;

	mitem = gtk_menu_item_new_with_label("Open new model...");
	gtk_signal_connect(GTK_OBJECT(mitem), "activate",
		GTK_SIGNAL_FUNC(interface_open_cb), model);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_object_set_data(GTK_OBJECT(menu), "model", model);
	gtk_widget_show(mitem);

	mitem = gtk_menu_item_new_with_label("Select background color...");
	gtk_signal_connect(GTK_OBJECT(mitem), "activate",
		GTK_SIGNAL_FUNC(interface_color_cb), model);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_check_menu_item_new_with_label("Wireframe");
	gtk_signal_connect(GTK_OBJECT(mitem), "toggled",
		GTK_SIGNAL_FUNC(interface_togglemode_cb), model);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_check_menu_item_new_with_label("Specular lighting");
	gtk_signal_connect(GTK_OBJECT(mitem), "toggled",
		GTK_SIGNAL_FUNC(interface_togglespecular_cb), model);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mitem),
		model->glflags & G3D_FLAG_GL_SPECULAR);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_check_menu_item_new_with_label("Shininess");
	gtk_signal_connect(GTK_OBJECT(mitem), "toggled",
		GTK_SIGNAL_FUNC(interface_toggleshininess_cb), model);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mitem),
		model->glflags & G3D_FLAG_GL_SHININESS);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_check_menu_item_new_with_label("Two-sided faces");
	gtk_signal_connect(GTK_OBJECT(mitem), "toggled",
		GTK_SIGNAL_FUNC(interface_toggletwosided_cb), model);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mitem),
		model->glflags & G3D_FLAG_GL_ALLTWOSIDE);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_check_menu_item_new_with_label("Textures");
	gtk_signal_connect(GTK_OBJECT(mitem), "toggled",
		GTK_SIGNAL_FUNC(interface_toggletextures_cb), model);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mitem), TRUE);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_menu_item_new_with_label("Info");
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_signal_connect_object(GTK_OBJECT(mitem), "activate",
		GTK_SIGNAL_FUNC(gtk_widget_show),
		(gpointer)model->interface.infowin);
	gtk_widget_show(mitem);

	mitem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	mitem = gtk_menu_item_new_with_label("Quit");
	gtk_signal_connect_object(GTK_OBJECT(mitem), "activate",
		GTK_SIGNAL_FUNC(gtk_main_quit), (gpointer)model->interface.window);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	return menu;
}

void interface_load(GtkFileSelection *selector, gpointer user_data)
{
	const char *filename = gtk_file_selection_get_filename(selector);
	G3DModel *model = (G3DModel*)gtk_object_get_data(GTK_OBJECT(selector),
		"model");
#if DEBUG > 0
	g_printerr("interface_load: loading %s ...\n", filename);
#endif
	object_clearmodel(model);
#if DEBUG > 2
	g_printerr("cleared old model data...\n");
#endif
	g3d_plugins_load_model(filename, model);
}

void interface_changefiletype_cb(GtkWidget *menuitem, gpointer user_data)
{
	GtkFileSelection *dialog = (GtkFileSelection*)user_data;
	GtkOptionMenu *omenu = (GtkOptionMenu*)gtk_object_get_data(
		GTK_OBJECT(menuitem), "omenu");
	int index = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(menuitem),
		"index"));
	char *extension = (char*)gtk_object_get_data(GTK_OBJECT(menuitem),
		"extension");
	gtk_option_menu_set_history(omenu, index);
	if(extension != NULL)
	{
		char pattern[256];
		g_snprintf(pattern, 256, "*.%s", extension);
		gtk_file_selection_complete(dialog, pattern);
	}
	else gtk_file_selection_complete(dialog, "*");
}

int interface_showopendialog(G3DModel *model)
{
	GtkWidget *dialog, *omenu, *menu, *mitem, *hbox, *label;
	GSList *plist;
	int index = 0;
	char name[256];

	dialog = gtk_file_selection_new("open 3D object file...");

	gtk_signal_connect_object(
		GTK_OBJECT(GTK_FILE_SELECTION(dialog)->ok_button),
		"clicked", GTK_SIGNAL_FUNC(interface_load), (gpointer)dialog);
	gtk_object_set_data(GTK_OBJECT(dialog), "model", model);

	gtk_signal_connect_object_after(
		GTK_OBJECT(GTK_FILE_SELECTION(dialog)->ok_button),
		"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);
	gtk_signal_connect_object_after(
		GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
		"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);

	omenu = gtk_option_menu_new();
	menu = gtk_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);

	mitem = gtk_menu_item_new_with_label("all files");
	gtk_object_set_data(GTK_OBJECT(mitem), "index", GINT_TO_POINTER(index));
	gtk_object_set_data(GTK_OBJECT(mitem), "omenu", omenu);
	gtk_signal_connect(GTK_OBJECT(mitem), "activate",
		GTK_SIGNAL_FUNC(interface_changefiletype_cb), dialog);
	gtk_menu_append(GTK_MENU(menu), mitem);
	gtk_widget_show(mitem);

	plist = g3d_plugins_getlist(G3D_PLUGIN_IMPORT);
	while(plist != NULL)
	{
		G3DPlugin *plugin = (G3DPlugin*)plist->data;
		gchar **ext = plugin->extensions;

		while(*ext != NULL)
		{
			index++;
			g_snprintf(name, 256, "*.%s (%s)", *ext, plugin->name);
			mitem = gtk_menu_item_new_with_label(name);
			gtk_object_set_data(GTK_OBJECT(mitem), "extension", *ext);
			gtk_object_set_data(GTK_OBJECT(mitem), "omenu", omenu);
			gtk_object_set_data(GTK_OBJECT(mitem), "index",
				GINT_TO_POINTER(index));
			gtk_signal_connect(GTK_OBJECT(mitem), "activate",
				GTK_SIGNAL_FUNC(interface_changefiletype_cb), dialog);
			gtk_menu_append(GTK_MENU(menu), mitem);
			gtk_widget_show(mitem);
			ext++;
		}
		plist = plist->next;
	}

	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), 0);

	hbox = gtk_hbox_new(TRUE, 0);
	label = gtk_label_new("show only these files:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), omenu, TRUE, TRUE, 0);
	gtk_widget_show(label);
	gtk_widget_show(hbox);

	gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(dialog)->action_area),
		hbox, FALSE, TRUE, 2);
	gtk_widget_show(omenu);

	gtk_widget_show(dialog);

	return EXIT_SUCCESS;
}

void interface_update_window(G3DModel *model)
{
	GtkWidget *widget;

	widget = model->interface.glarea;

	gtk_widget_queue_draw_area(widget, 0, 0,
		widget->allocation.width, widget->allocation.height);
}

/*****************************************************************************/

void g3d_interface_update(void)
{
	while (gtk_events_pending()) gtk_main_iteration();
}

void g3d_interface_progress_init(G3DModel *model)
{
	if(model->interface.show_statusbar == 0)
	{
		gtk_widget_show(model->interface.statushbox);
	}
	gtk_progress_bar_update(GTK_PROGRESS_BAR(model->interface.progressbar),
		0.0);
}

void g3d_interface_progress_update(G3DModel *model, gfloat percentage)
{
	gtk_progress_bar_update(GTK_PROGRESS_BAR(model->interface.progressbar),
		percentage);
	g3d_interface_update();
}

void g3d_interface_progress_finish(G3DModel *model)
{
	if(model->interface.show_statusbar == 0)
	{
		gtk_widget_hide(model->interface.statushbox);
	}
}

