/*
 * $Id: main.c,v 1.5 2005/07/31 17:49:17 dahms Exp $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#ifdef HAVE_GTKGLEXT
#	include <gtk/gtkgl.h>
#endif

#include <g3dviewer_plugin.h>

#include "main.h"
#include "interface.h"
#include "plugins.h"
#include "trackball.h"

static gboolean parse_only = FALSE;

int main_parseargs(int *argc, char ***argv, G3DModel *model);

int main(int argc, char **argv)
{
	G3DModel *model;

	gtk_init(&argc, &argv);
#ifdef HAVE_GTKGLEXT
	gtk_gl_init(&argc, &argv);
#endif

	model = g_new0(G3DModel, 1);
	model->zoom = 45;
	model->mouse.beginx = 0;
	model->mouse.beginy = 0;
	model->filename = NULL;
	model->bgcolor[0] = 0.9;
	model->bgcolor[1] = 0.8;
	model->bgcolor[2] = 0.6;
	model->bgcolor[3] = 1.0;
	model->glflags =
		/* G3D_FLAG_GL_SPECULAR | */
		G3D_FLAG_GL_SHININESS |
		G3D_FLAG_GL_ALLTWOSIDE |
		G3D_FLAG_GL_TEXTURES;

	main_parseargs(&argc, &argv, model);

	trackball(model->quat, 0.0, 0.0, 0.0, 0.0);

	plugins_init();
	interface_init(model);

	if(model->filename != NULL)
		g3d_plugins_load_model(model->filename, model);
	else
		interface_showopendialog(model);

	if(parse_only)
		return EXIT_SUCCESS;

	interface_update_window(model);

	gtk_main();

	gtk_exit(0);

	return EXIT_SUCCESS;
}

void main_showhelp(void)
{
	printf(
		"g3dviewer - a 3D model viewer\n"
		"\n"
		"usage: g3dviewer [--option ...] [<filename>]\n"
		"where option can be:\n"
		"  --help               show this help screen\n"
		);
	exit(1);
}

int main_parseargs(int *argc, char ***argv, G3DModel *model)
{
	/* program name */
	(*argc)--;
	(*argv)++;
	while(*argc > 0)
	{
#if DEBUG > 3
		g_printerr("arg: %s\n", **argv);
#endif
		if(strcmp(**argv, "--help") == 0) main_showhelp();
		else if(strcmp(**argv, "--parse-only") == 0)
		{
			parse_only = TRUE;
		}
		else
		{
			model->filename = **argv;
		}
		(*argv)++;
		(*argc)--;
	}

	return EXIT_SUCCESS;
}

