#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <stdlib.h>

#include <g3dviewer_plugin.h>

#include "gl.h"
#include "image.h"

G3DImage *g3d_texture_load_cached(G3DModel *model, const gchar *filename)
{
	G3DImage *image;
#ifdef HAVE_GTK20
	gchar *basename;
#endif
#if DEBUG > 2
	gchar *ppmname;
#endif

	if(model->texture_files == NULL)
	{
		model->texture_files = g_hash_table_new(g_str_hash, g_str_equal);
	}

	image = g_hash_table_lookup(model->texture_files, filename);

	if(image != NULL)
	{
		return image;
	}

	image = g_new0(G3DImage, 1);
	if(g3d_plugins_load_image(filename, image) == EXIT_SUCCESS)
	{
		g_hash_table_insert(model->texture_files, (gpointer)filename, image);
#if DEBUG > 2
		ppmname = g_strdup_printf("/tmp/%s.ppm", filename);
		g3d_image_dump_ppm(image, ppmname);
		g_free(ppmname);
#endif
	}
	else
	{
#ifdef HAVE_GTK20
		/* try to load file without path */
		basename = g_path_get_basename(filename);
		if(g3d_plugins_load_image(basename, image) == EXIT_SUCCESS)
		{
			g_hash_table_insert(model->texture_files, (gpointer)filename,
				image);
		}
		else
		{
#endif
			g_free(image);
			image = NULL;
#ifdef HAVE_GTK20
		}
		g_free(basename);
#endif
	}

	if(image)
	{
		image->scaleu = 1.0;
		image->scalev = 1.0;
	}

	return image;
}

int g3d_texture_load_all_textures(G3DModel *model)
{
#if 0
	G3DMaterial *material;
	GSList *mitem;
#endif
	if(model->texture_files != NULL)
		g_hash_table_foreach(model->texture_files,
			gl_load_texture, NULL);

#if 0
	mitem = model->materials;
	while(mitem)
	{
		material = (G3DMaterial *)mitem->data;
		if(material->texture)
		{
			gl_load_texture(NULL, material->texture, NULL);
		}
		mitem = mitem->next;
	}
#endif

	return EXIT_SUCCESS;
}

int g3d_texture_prepare(G3DImage *texture)
{
	guint w = 1, h = 1;

	while(w < texture->width) w *= 2;
	while(h < texture->height) h *= 2;

	if(w < h) w = h;
	else if(h < w) h = w;

	if((h == texture->height) && (w == texture->width)) return EXIT_SUCCESS;

	texture->scaleu = (GLfloat)texture->width / (GLfloat)w;
	texture->scalev = (GLfloat)texture->height / (GLfloat)h;

#if DEBUG > 0
	g_printerr("resizing texture to %ux%u (%f, %f)\n", w, h,
		texture->scaleu, texture->scalev);
#endif

	return image_expand(texture, w, h);
}


static gboolean remove_teximg(gpointer key, gpointer value, gpointer data)
{
	G3DImage *image = (G3DImage *)value;

	if(image->name)
		g_free(image->name);
	if(image->pixeldata)
		g_free(image->pixeldata);
	g_free(image);

	return TRUE;
}

void g3d_texture_free_cache(G3DModel *model)
{
	if(model->texture_files != NULL)
		g_hash_table_foreach_remove(model->texture_files, remove_teximg, NULL);
}
